import binascii

def to_guid(bytes):
    """Converts a 16 byte string to a GUID"""
    if bytes is None:
        return None
        
    s = binascii.hexlify(bytes)
    parts = [s[6:8] + s[4:6] + s[2:4] + s[0:2], s[10:12] + s[8:10], s[14:16] + s[12:14], s[16:20], s[20:32]]
    
    return "{" + '-'.join(parts) + "}"


from __future__ import print_function
import copy, re, string

AnsiAttributes = {
    'clear': 0,
    'reset': 0,
    'bold': 1,
    'dark': 2,
    'underline': 4,
    'underscore': 4,
    'blink': 5,
    'reverse': 7,
    'concealed': 8,
    'black': 30,
    'red': 31,
    'green': 32,
    'yellow': 33,
    'blue': 34,
    'magenta': 35,
    'cyan': 36,
    'white': 37,
    'on_black': 40,
    'on_red': 41,
    'on_green': 42,
    'on_yellow': 43,
    'on_blue': 44,
    'on_magenta': 45,
    'on_cyan': 46,
    'on_white': 47,
}

class UiColor(object):
    """Mixin that provides access to functions related to color output"""
    # A class that inherits UiColor must implement the following methods:
    #
    # print - function that outputs text to an underlying stream
    # supports_color - property (bool) that returns if the stream supports color
    #

    def __init__(self, *args, **kwargs):
        pass

    def ansi(self, *args):
        global AnsiAttributes
        attr = ";".join(map(str, [AnsiAttributes[x] for x in args]))
        if len(attr) != 0:
            attr = "\x1B[{}m".format(attr)
        return attr

    def colorize(self, *args):
        try:
            if self.supports_color == True:
                return self.ansi(*args)
            else:
                return ''
        except Exception as e:
            return ''

    def substitute_colors(self, msg, in_prompt = None):
        _str = copy.copy(msg)
        pre_color = post_color = ''
        if (in_prompt):
            pre_color = "\x01"
            post_color = "\x02"
        _str = re.sub(r'%cya', pre_color + self.colorize('cyan') + post_color, _str)
        _str = re.sub(r'%red', pre_color + self.colorize('red') + post_color, _str)
        _str = re.sub(r'%grn', pre_color + self.colorize('green') + post_color, _str)
        _str = re.sub(r'%blu', pre_color + self.colorize('blue') + post_color, _str)
        _str = re.sub(r'%yel', pre_color + self.colorize('yellow') + post_color, _str)
        _str = re.sub(r'%whi', pre_color + self.colorize('white') + post_color, _str)
        _str = re.sub(r'%mag', pre_color + self.colorize('magenta') + post_color, _str)
        _str = re.sub(r'%blk', pre_color + self.colorize('black') + post_color, _str)
        _str = re.sub(r'%dred', pre_color + self.colorize('dark','red') + post_color, _str)
        _str = re.sub(r'%dgrn', pre_color + self.colorize('dark','green') + post_color, _str)
        _str = re.sub(r'%dblu', pre_color + self.colorize('dark','blue') + post_color, _str)
        _str = re.sub(r'%dyel', pre_color + self.colorize('dark','yellow') + post_color, _str)
        _str = re.sub(r'%dcya', pre_color + self.colorize('dark','cyan') + post_color, _str)
        _str = re.sub(r'%dwhi', pre_color + self.colorize('dark','white') + post_color, _str)
        _str = re.sub(r'%dmag', pre_color + self.colorize('dark','magenta') + post_color, _str)
        _str = re.sub(r'%und', pre_color + self.colorize('underline') + post_color, _str)
        _str = re.sub(r'%bld', pre_color + self.colorize('bold') + post_color, _str)
        _str = re.sub(r'%clr', pre_color + self.colorize('clear') + post_color, _str)
        _str = re.sub(r'%bgblu', pre_color + self.colorize('on_blue') + post_color, _str)
        _str = re.sub(r'%bgyel', pre_color + self.colorize('on_yellow') + post_color, _str)
        _str = re.sub(r'%bggrn', pre_color + self.colorize('on_green') + post_color, _str)
        _str = re.sub(r'%bgmag', pre_color + self.colorize('on_magenta') + post_color, _str)
        _str = re.sub(r'%bgblk', pre_color + self.colorize('on_black') + post_color, _str)
        _str = re.sub(r'%bgred', pre_color + self.colorize('on_red') + post_color, _str)
        _str = re.sub(r'%bgcyn', pre_color + self.colorize('on_cyan') + post_color, _str)
        _str = re.sub(r'%bgwhi', pre_color + self.colorize('on_white') + post_color, _str)

        return _str

    def reset_color(self):
        if not self.supports_color:
            return
            
        self.print(self.colorize('clear'))

    def do_colorize(self, *args):
        try:
            if self.supports_color == True:
                return self.ansi(*args)
            else:
                return ''
        except Exception as e:
            return ''





















import struct

ENDIAN_LITTLE = 0
ENDIAN_BIG = 1

# Python 2/3 compatibility
try:
    from itertools import izip_longest
except ImportError:
    from itertools import zip_longest as izip_longest

def to_num(string, wrap = 60):
    code = struct.unpack("{}B".format(len(string)), string)
    buff = ""
    for byte in range(0, len(code)):
        if (byte % 15 == 0) and (len(buff) > 0):
            buff += "\r\n"
        buff += '0x{:02x}, '.format(code[byte])
    buff = buff.rstrip(', ')
    buff += "\r\n"
    return buff
    
def to_dword(string, wrap = 60):
    def each_slice(iterable, n, fillvalue = None):
        args = [iter(iterable)] * n
        return izip_longest(fillvalue = fillvalue, *args)
    
    code = string
    alignnr = len(string) % 4
    if (alignnr > 0):
        code += "\x00" * (4 - alignnr)
    codevalues = []
    for chars4 in each_slice(list(code), 4):
        chars4 = "".join(chars4)
        dwordvalue = struct.unpack("<L", chars4)
        codevalues.append(dwordvalue[0])
    buff = ""
    for byte in range(0, len(codevalues)):
        if (byte % 8 == 0) and (len(buff) > 0):
            buff += "\r\n"
        buff += '0x{:08x}, '.format(codevalues[byte])
    buff = buff.rstrip(', ')
    buff += "\r\n"
    return buff
    
def to_unescape(data, endian = ENDIAN_LITTLE, prefix = '%u'):
    if (len(data) % 2 != 0):
        data += "\x41"
    
    dptr = 0
    buff = ''
    while (dptr < len(data)):
        c1 = struct.unpack("{}B".format(len(data[dptr : dptr + 1])), data[dptr : dptr + 1])[0]
        dptr += 1
        c2 = struct.unpack("{}B".format(len(data[dptr : dptr + 1])), data[dptr : dptr + 1])[0]
        dptr += 1
        
        if (endian == ENDIAN_LITTLE):
            buff += "{}{:02x}{:02x}".format(prefix, c2, c1)
        else:
            buff += "{}{:02x}{:02x}".format(prefix, c1, c2)
            
    return buff

def pack_int64le(val):
    return struct.pack("<Q", val)
    
def ror(val, cnt):
    tmpbits = struct.pack(">L", val)
    bits = list(''.join('{0:08b}'.format(ord(x), 'b') for x in tmpbits))
    for c in range(1, cnt + 1):
        bits.insert(0, bits.pop())
        
    newbits = ''.join(str(x) for x in bits)
    tmpint = int(newbits, 2)
    tmpbytes = bytearray()
    while tmpint:
        tmpbytes.append(tmpint & 0xff)
        tmpint >>= 8
    
    retval = bytes(tmpbytes[::-1])
    while len(retval) < 4:
        retval = "\x00" + retval
    
    return struct.unpack(">L", retval)[0]
    
def rol(val, cnt):
    tmpbits = struct.pack(">L", val)
    bits = list(''.join('{0:08b}'.format(ord(x), 'b') for x in tmpbits))
    for c in range(1, cnt + 1):
        bits.append(bits.pop(0))
        
    newbits = ''.join(str(x) for x in bits)
    tmpint = int(newbits, 2)
    tmpbytes = bytearray()
    while tmpint:
        tmpbytes.append(tmpint & 0xff)
        tmpint >>= 8
    
    retval = bytes(tmpbytes[::-1])
    while len(retval) < 4:
        retval = "\x00" + retval
    
    return struct.unpack(">L", retval)[0]


import hashlib

def md5_raw(string):
    """Returns the MD5 digest of the provided string as raw binary"""
    m = hashlib.md5()
    m.update(string)
    return m.digest()
    
def md5(string):
    """Returns the MD5 digest of the provided string as a hexadecimal string"""
    m = hashlib.md5()
    m.update(string)
    return m.hexdigest()
    
def sha1_raw(string):
    """Returns the SHA1 digest of the provided string as raw binary"""
    s = hashlib.sha1()
    s.update(string)
    return s.digest()
    
def sha1(string):
    """Returns the SHA1 digest of the provided string as a hexadecimal string"""
    s = hashlib.sha1()
    s.update(string)
    return s.hexdigest()

def sha256_raw(string):
    """Returns the SHA256 digest of the provided string as raw binary"""
    s = hashlib.sha256()
    s.update(string)
    return s.digest()

def sha256(string):
    """Returns the SHA256 digest of the provided string as a hexadecimal string"""
    s = hashlib.sha256()
    s.update(string)
    return s.hexdigest()


try:
    import hex
except ImportError:
    from . import hex
try:
    import HTMLParser
except ImportError:
    import html.parser as HTMLParser
import random
import re
import struct

def uri_encode(string, mode = 'hex-normal'):
    """Encodes a string for use in HTTP URIs and parameters"""
    if string is None:
        return ""
        
    if mode == 'none':
        return string
        
    all = r'.'
    noslashes = r'[^\/\\]+'
    normal = r'[^a-zA-Z0-9\/\\\.\-_~]+'
    
    if mode == 'hex-all':
        return re.sub(all, lambda s: hex.to_hex(s.group(), '%'), string)
    elif mode == 'hex-normal':
        return re.sub(normal, lambda s: hex.to_hex(s.group(), '%'), string)
    elif mode == 'hex-noslashes':
        return re.sub(noslashes, lambda s: hex.to_hex(s.group(), '%'), string)
    elif mode == 'hex-random':
        res = ''
        for c in bytearray(string, "utf-8"):
            b = chr(c)
            if (random.randrange(2) == 0):
                b = re.sub(all, lambda s: hex.to_hex(s.group(), '%'), b)
                res += b
            else:
                b = re.sub(normal, lambda s: hex.to_hex(s.group(), '%'), b)
                res += b
        return res
    else:
        raise ValueError("Invalid mode '{}'".format(mode))

def html_encode(string, mode = 'hex'):
    """Encodes a string for use in HTTP URIs and parameters"""
    if mode == 'hex':
        return "".join(map(lambda i: "&#x" + "{:02x}".format(i) + ";", struct.unpack('{}B'.format(len(string)), string)))
    elif mode == 'int':
        return "".join(map(lambda i: "&#" + str(i) + ";", struct.unpack('{}B'.format(len(string)), string)))
    elif mode == 'int-wide':
        return "".join(map(lambda i: "&#" + ("0" * (7 - len(str(i)))) + str(i) + ";", struct.unpack('{}B'.format(len(string)), string)))
    else:
        raise ValueError("Invalid mode '{}'".format(mode))

def html_decode(string):
    """Decodes an HTML encoded string"""
    return HTMLParser.HTMLParser().unescape(string)




def to_octal(string, prefix = "\\"):
    """Returns the octal representation of a string"""
    octal = ""
    
    for b in bytearray(string):
        octal = octal + "{}{:o}".format(prefix, b)
        
    return octal


import re
try:
    import StringIO
except ImportError:
    import io as StringIO

try:
    import zlib
    zlib_present = True
except ImportError:
    zlib_present = False
    
try:
    import gzip as Gzip
    gzip_present = True
except ImportError:
    gzip_present = False
    

def compress(string):
    """Compresses a string by removing newline characters and extra whitespace"""
    if re.sub(r'\n', ' ', string) != string:
        string = re.sub(r'\n', ' ', string)
        
    if re.sub(r'\s+', ' ', string) != string:
        string = re.sub(r'\s+', ' ', string)
        
    if re.sub(r'^\s+', '', string) != string:
        string = re.sub(r'^\s+', '', string)
        
    if re.sub(r'\s+$', '', string) != string:
        string = re.sub(r'\s+$', '', string)
        
    return string
    
def zlib_deflate(string, level = zlib.Z_BEST_COMPRESSION):
    """Compresses a buffer using Zlib"""
    if zlib_present == True:
        zstream = zlib.compressobj(level)
        buf = zstream.compress(string)
        buf += zstream.flush(zlib.Z_FINISH)
        return buf
    else:
        raise RuntimeError("Zlib support is not present.")
        
def zlib_inflate(string):
    """Decompresses a buffer using Zlib"""
    if zlib_present == True:
        zstream = zlib.decompressobj()
        buf = zstream.decompress(string)
        buf += zstream.flush()
        return buf
    else:
        raise RuntimeError("Zlib support is not present.")
        
def gzip(string, level = 9):
    """Compresses a buffer using Gzip"""
    if gzip_present == False:
        raise RuntimeError("Gzip support is not present.")
        
    if (level < 1 or level > 9):
        raise RuntimeError("Invalid Gzip compression level")
        
    s = StringIO.StringIO("")
    
    gz = Gzip.GzipFile(filename = None, mode = 'wb', compresslevel = level, fileobj = s)
    gz.write(string)
    gz.close()
    
    return s.getvalue()
    
def ungzip(string):
    """Decompresses a buffer using Gzip"""
    if gzip_present == False:
        raise RuntimeError("Gzip support is not present.")
        
    s = ""
    if hasattr(s, "encode") == True:
        s.encode('ascii', 'ignore')
        
    gz = Gzip.GzipFile(filename = None, mode = 'rb', fileobj = StringIO.StringIO(string))
    s += gz.read()
    gz.close()
    return s


try:
    import basex
except ImportError:
    from . import basex
import random
import re
import struct

def to_unicode(string = '', type = 'utf-16le', mode = '', size = ''):
    """Converts ASCII text into a Unicode string"""
    def utf7_block(a):
        out = ''
        if a != '+':
            out = re.sub(r'[=\r\n]', '', basex.encode_base64(to_unicode(a, 'utf-16be')))
        return '+' + out + '-'
 
    if string is None:
        return ''
        
    if type == 'utf-16le':
        return struct.pack('<{}H'.format(len(string)), *struct.unpack('{}B'.format(len(string)), string))
    elif type == 'utf-16be':
        return struct.pack('>{}H'.format(len(string)), *struct.unpack('{}B'.format(len(string)), string))
    elif type == 'utf-32le':
        return struct.pack('<{}L'.format(len(string)), *struct.unpack('{}B'.format(len(string)), string))
    elif type == 'utf-32be':
        return struct.pack('>{}L'.format(len(string)), *struct.unpack('{}B'.format(len(string)), string))
    elif type == 'utf-7':
        if mode == 'all':
            return re.sub(r'.', lambda a: utf7_block(a.group()), string)
        else:
            return re.sub(r'[^\n\r\t\ A-Za-z0-9\'\(\),-.\/\:\?]', lambda a: utf7_block(a.group()), string)
    elif type == 'utf-8':
        if size == '':
            size = 2
            
        if size >= 2 and size <= 7:
            _string = ''
            for a in bytearray(string):
                if (a < 21 or a > 0x7f) or mode != '':
                    # try not to cringe
                    tmp = struct.pack('B', a)
                    bin = ''.join('{0:08b}'.format(ord(bites), 'b') for bites in tmp)
                    bin = list(bin)
                    bin = list(map(int, bin))
                    
                    out = [0] * (8 * size)
                    
                    for i in range(0, size):
                        out[i] = 1
                        out[i * 8] = 1
                        
                    i = 0
                    byte = 0
                    
                    for bit in reversed(bin):
                        while True:
                            if i < 6:
                                mod = (((size * 8) - 1) - byte * 8) - i
                                out[mod] = bit
                                i = i + 1
                            else:
                                byte = byte + 1
                                i = 0
                                continue
                            break
                            
                    if mode != '':
                        if mode == 'overlong':
                            pass
                        elif mode == 'invalid':
                            done = 0
                            while done == 0:
                                bits = [7, 8, 15, 16, 23, 24, 31, 32, 41]
                                for bit in bits:
                                    bit = (size * 8) - bit
                                    if bit > 1:
                                        _set = random.randrange(2)
                                        if out[bit] != _set:
                                            out[bit] = _set
                                            done = 1
                        else:
                            raise TypeError('Invalid mode.  Only "overlong" and "invalid" are accepted modes for utf-8')
                    newbin = ''.join(str(x) for x in out)
                    tmpint = int(newbin, 2)
                    tmpbytes = bytearray()
                    while tmpint:
                        tmpbytes.append(tmpint & 0xff)
                        tmpint >>= 8
                    _string += bytes(tmpbytes[::-1])
                else:
                    _string += struct.pack('B', a)
            return _string
        else:
            raise TypeError('invalid utf-8 size')


import setuptools

with open("README.md", "r") as fd:
    long_description = fd.read()

setuptools.setup(
    name="paynspray",
    version="0.2.2",
    author="The Munshi Group",
    author_email="support@munshigroup.com",
    description="Provides access to functions pertaining to text generation, manipulation, & presentation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/munshigroup/paynspray",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


import re

def dogsay(text, width = 39):
    """Returns an ASCII image of a dog saying the provided text"""
    text_lines = re.findall(".{1,%d}" % (width - 4), str(text))
    lengths = map(lambda a: len(a), text_lines)
    lengths.sort()
    max_length = int(lengths[len(lengths) - 1])
    cloud_parts = []
    cloud_parts.append(" {}".format(('_' * (max_length + 2))))
    if len(text_lines) == 1:
        cloud_parts.append("< {} >".format(text))
    else:
        cloud_parts.append("/ {} \\".format(str(text_lines[0]).ljust(max_length, ' ')))
        if len(text_lines) > 2:
            for line in text_lines[1:(1 + (len(text_lines) - 2))]:
                cloud_parts.append("| {} |".format(str(line).ljust(max_length, ' ')))
        cloud_parts.append("\\ {} /".format(str(text_lines[len(text_lines) - 1]).ljust(max_length, ' ')))
    cloud_parts.append(" {}".format(('-' * (max_length + 2))))
    
    doggo = """     \\     .--.
      \\  _/ee/ \\
        (T_, \\_/      ,
           )="`------.))
          o|      /   /
            \\  /__\\  (\\
            |\\ \\  _/ / \\
           (_(_/ (__/(_/"""
    cloud_parts.append(doggo)
    return "\n".join(cloud_parts)